@@ -46,9 +46,9 @@ Codec <- R6Class("Codec", inherit = ArrowObject,
 )
 Codec$create <- function(type = "gzip", compression_level = NA) {
   if (is.string(type)) {
-    type <- shared_ptr(Codec, util___Codec__Create(
+    type <- util___Codec__Create(
       compression_from_name(type), compression_level
-    ))
+    )
   }
   assert_is(type, "Codec")
   type
@@ -63,6 +63,8 @@ Codec$create <- function(type = "gzip", compression_level = NA) {
 #' "zstd", "lz4", "lzo", or "bz2", case insensitive.
 #' @return Logical: is `type` available?
 #' @export
+#' @examplesIf arrow_available()
+#' codec_is_available("gzip")
 codec_is_available <- function(type) {
   util___Codec__IsAvailable(compression_from_name(type))
 }
@@ -97,13 +99,13 @@ compression_from_name <- function(name) {
 #' @export
 #' @include arrow-package.R
 CompressedOutputStream <- R6Class("CompressedOutputStream", inherit = OutputStream)
-CompressedOutputStream$create <- function(stream, codec = "gzip", compression_level = NA){
+CompressedOutputStream$create <- function(stream, codec = "gzip", compression_level = NA) {
   codec <- Codec$create(codec, compression_level = compression_level)
   if (is.string(stream)) {
     stream <- FileOutputStream$create(stream)
   }
   assert_is(stream, "OutputStream")
-  shared_ptr(CompressedOutputStream, io___CompressedOutputStream__Make(codec, stream))
+  io___CompressedOutputStream__Make(codec, stream)
 }
 
 #' @rdname compression
@@ -111,11 +113,11 @@ CompressedOutputStream$create <- function(stream, codec = "gzip", compression_le
 #' @format NULL
 #' @export
 CompressedInputStream <- R6Class("CompressedInputStream", inherit = InputStream)
-CompressedInputStream$create <- function(stream, codec = "gzip", compression_level = NA){
+CompressedInputStream$create <- function(stream, codec = "gzip", compression_level = NA) {
   codec <- Codec$create(codec, compression_level = compression_level)
   if (is.string(stream)) {
     stream <- ReadableFile$create(stream)
   }
   assert_is(stream, "InputStream")
-  shared_ptr(CompressedInputStream, io___CompressedInputStream__Make(codec, stream))
+  io___CompressedInputStream__Make(codec, stream)
 }
